@@ -1,7 +1,8 @@
 
 ##  RPushbullet -- R interface to Pushbullet libraries
 ##
-##  Copyright (C) 2014  Dirk Eddelbuettel <edd@debian.org>
+##  Copyright (C) 2014         Dirk Eddelbuettel <edd@debian.org>
+##  Copyright (C) 2014 - 2015  Dirk Eddelbuettel and Mike Birdgeneau
 ##
 ##  This file is part of RPushbullet.
 ##
@@ -20,7 +21,7 @@
 
 
 ##' This function posts a message to Pushbullet. Different types of
-##' messages are supported: \sQuote{note}, \sQuote{link}, 
+##' messages are supported: \sQuote{note}, \sQuote{link},
 ##' \sQuote{address}, or \sQuote{file}.
 ##'
 ##' This function invokes the \sQuote{pushes} functionality of
@@ -31,7 +32,7 @@
 ##' title and body of the note.  If a \sQuote{link} is pushed, the recipient's web
 ##' browser is opened at the given URL.  If an \sQuote{address} is
 ##' pushed, the recipient's web browser is opened in map mode at the
-##' given address. 
+##' given address.
 ##'
 ##' If \sQuote{recipients} argument is missing, the post is pushed to
 ##' \emph{all} devices in accordance with the API definition. If
@@ -39,7 +40,7 @@
 ##' names (from either the config file or a corresponding
 ##' option). Lastly, if \sQuote{recipients} is a numeric vector, the
 ##' post is pushed the corresponding elements in the devices vector.
-##' 
+##'
 ##' In other words, the default of value of no specified recipients results
 ##' in sending to all devices. If you want a particular subset of
 ##' devices you have to specify it name or index. A default device can be set
@@ -61,13 +62,16 @@
 ##' \code{type} is \sQuote{file}), defaults to \dQuote{text/plain}.
 ##' @param recipients A character or numeric vector indicating the
 ##' devices this post should go to. If missing, the default device
-##' is looked up from an optional setting, and if none has been set 
+##' is looked up from an optional setting, and if none has been set
 ##' the push is sent to all devices.
-##' @param email An alternative way to specify a recipient is to specify 
+##' @param email An alternative way to specify a recipient is to specify
 ##' an email address. If both \code{recipients} and \code{email} are
 ##' present, \code{recipients} is used.
+##' @param channel A channel tag used to specify the name of the channel
+##' as the recipient. If either \code{recipients} or \code{email} are present,
+##' they will take precedence over \code{channel}.
 ##' @param deviceind (Deprecated) The index (or a vector/list of indices) of the
-##' device(s) in the list of devices. 
+##' device(s) in the list of devices.
 ##' @param apikey The API key used to access the service. It can be
 ##' supplied as an argument here, via the global option
 ##' \code{rpushbullet.key}, or via the file \code{~/.rpushbullet.json}
@@ -78,20 +82,21 @@
 ##' \code{~/.rpushbullet.json} which is read at package
 ##' initialization.
 ##' @param verbose Boolean switch to add additional output
+##' @param debug Boolean switch to add even more debugging output
 ##' @return A JSON result record is return invisibly
 ##' @author Dirk Eddelbuettel
 ##' @examples
 ##' \dontrun{
 ##' # A note
 ##' pbPost("note", "A Simple Test", "We think this should work.\nWe really do.")
-##' 
+##'
 ##' # An address -- should open browser in Google Maps
 ##' pbPost(type="address", title="An Address", body="South Pole, Antarctica")
-##' 
+##'
 ##' # A URL -- should open browser
-##' pbPost(type="link", title="Some title", body="Some URL", 
+##' pbPost(type="link", title="Some title", body="Some URL",
 ##'        url="http://cran.r-project.org/package=RPushbullet")
-##'  
+##'
 ##' # A file
 ##' pbPost(type="file", url=system.file("DESCRIPTION", package="RPushbullet"))
 ##' }
@@ -104,26 +109,34 @@ pbPost <- function(type=c("note", "link", "address", "file"),
                    filetype="text/plain",# file type for upload of type='file'
                    recipients,           # devices to post to
                    email,                # alternatively use an email
+                   channel,              # alternatively specify a channel
                    deviceind,            # deprecated, see detail
                    apikey = .getKey(),
                    devices = .getDevices(),
-                   verbose = FALSE) {
+                   verbose = FALSE,
+                   debug = FALSE) {
 
     type <- match.arg(type)
 
     if (!missing(deviceind)) {
-        if (missing(recipients) && missing(email)) {
+        if (missing(recipients) && missing(email) && missing(channel)) {
             warning("Agument 'deviceind' is deprecated. Please use 'recipients'.", call.=FALSE)
             recipients <- deviceind
         } else {
-            warning("Using 'recipients' (or 'email') and ignoring deprecated 'deviceinds'.",
+            warning("Using 'recipients' (or 'email' or 'channel') and ignoring deprecated 'deviceinds'.",
                     call.=FALSE)
         }
     }
 
-    if (missing(recipients) && missing(email)) {
+    if (missing(recipients) && missing(email) && missing(channel)) {
+        if (debug) message("missing recipient and email and channel")
         recipients <- .getDefaultDevice() # either supplied, or 0 as fallback
-        dest <- match(recipients, .getNames())
+        if (recipients==0) {
+            dest <- ''
+        } else {
+            dest <- match(recipients, .getNames())
+        }
+        email <- channel <- NA
     } else {
         if (!missing(recipients)) {     # hence recipient present
             if (is.character(recipients)) {
@@ -131,14 +144,20 @@ pbPost <- function(type=c("note", "link", "address", "file"),
             } else {
                 dest <- recipients      # numeric values
             }
-        } else {                        # hence email presnt
-           dest <- email
+            email <- NA
+        } else {                        # either email or channel present
+            if(!missing(email)) {
+                dest <- email
+            } else {                    # hence channel present
+                dest <- channel
+                email <- NA # Set e-mail to NA, missing() is unreliable
+            }
        }
     }
-    
+
     pburl <- "https://api.pushbullet.com/v2/pushes"
     curl <- .getCurl()
-    
+
     ## if (is.null(deviceind))
     ##     deviceind <- .getDefaultDevice()
 
@@ -147,18 +166,18 @@ pbPost <- function(type=c("note", "link", "address", "file"),
     ##     ## explicitly including others would result in double-tapping
     ##     deviceind <- 0
     ## }
-    
+
     if (type=="file") {
         if (url != "" && filetype != "") {             # Request Upload
             url <- normalizePath(url)                  # abs/rel path, tilde expansion, ...
             uploadrequest <- .getUploadRequest(filename = url, filetype = filetype)
             #fileurl <- uploadrequest$file_url
-            
+
             # Upload File
             txt <- sprintf(paste0("%s -s -i %s -F awsaccesskeyid='%s' -F acl='%s' ",
                                   "-F key='%s' -F signature='%s' -F policy='%s' ",
                                   "-F content-type='%s' -F 'file=@%s'"),
-                           curl, 
+                           curl,
                            uploadrequest$upload_url,
                            uploadrequest$data['awsaccesskeyid'],
                            uploadrequest$data['acl'],
@@ -172,8 +191,13 @@ pbPost <- function(type=c("note", "link", "address", "file"),
     }
 
     ret <- lapply(dest, function(d) {
-        if (is.character(d)) {          # this was an email
-            tgt <- sprintf(' -d email="%s" ', d)
+        if (debug) message(sprintf("in lapply, d is: %s", d))   
+        if (is.character(d)) {          # this is an email or channel.
+            if (!is.na(email)){
+                tgt <- sprintf(' -d email="%s" ', d)
+            } else {                    # hence assume channel
+                tgt <- sprintf(' -d channel_tag="%s" ', d)
+            }
         } else if (is.numeric(d)) {     # this a listed device, now transfered to index
             tgt <- ifelse(d == 0,       # if zero, then use all devices
                           '',           # otherwise given specific device
@@ -181,7 +205,7 @@ pbPost <- function(type=c("note", "link", "address", "file"),
         } else {                        # fallback, should not get reached
             tgt <- ''
         }
-        
+
         txt <- switch(type,
 
                       ## curl https://api.pushbullet.com/v2/pushes \
@@ -212,13 +236,13 @@ pbPost <- function(type=c("note", "link", "address", "file"),
                       ##                pburl, apikey, device, title, body),
 
                       ## for file see docs, need to upload file first
-                      file = sprintf(paste0('%s -s -u %s: %s ',
+                      file = sprintf(paste0('%s -s %s -u %s: %s ',
                                             '-d type="file" -d file_name="%s" ',
                                             '-d file_type="%s" ',
                                             '-d file_url="%s" ',
                                             '-d body="%s" -X POST'),
-                                     curl, apikey, pburl, tgt,
-                                     basename(uploadrequest$file_name), 
+                                     curl, pburl, apikey, tgt,
+                                     basename(uploadrequest$file_name),
                                      uploadrequest$file_type, uploadrequest$file_url, body)
                       )
         if (verbose) print(txt)
